



pushd tests
IF %ERRORLEVEL% NEQ 0 exit /B 1
pushd pyboost
IF %ERRORLEVEL% NEQ 0 exit /B 1
popd
IF %ERRORLEVEL% NEQ 0 exit /B 1
popd
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\include\boost\python\python.hpp exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\include\boost\python exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\bin\boost_python311.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\lib\boost_python311.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\lib\libboost_python311.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\bin\boost_numpy311.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\lib\boost_numpy311.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\Library\lib\libboost_numpy311.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
